-module(bvMPriority).

-include("eBhv3.hrl").

-export([open/2, tick/2, close/2]).

-spec open(bt_node(), bt_state()) -> bt_state().
open(#{id := ID, children := Children} = _BTNode, BTState) ->
   bvBlackBoard:set(running_children, Children, ID, BTState).

-spec tick(bt_node(), bt_state()) -> {bt_status(), bt_state()}.
tick(#{id := ID} = _BTNode, BTState) ->
   RunningChildren = bvBlackBoard:get(running_children, ID, BTState),
   doTick(RunningChildren, ID, BTState).

-spec close(bt_node(), bt_state()) -> bt_state().
close(#{id := ID} = _BTNode, State) ->
   bvBlackBoard:remove(running_children, ID, State).

doTick([ChildID | T] = Children, ID, BTState) ->
   case bvBaseNode:execute(ChildID, BTState) of
      {?BT_FAILURE, BTState1} ->
         doTick(T, ID, BTState1);
      {?BT_RUNNING, BTState1} ->
         BTState2 = bvBlackBoard:set(running_children, Children, ID, BTState1),
         {?BT_RUNNING, BTState2};
      {BTStatus, BTState1} ->
         {BTStatus, BTState1}
   end;
doTick([], _ID, BTState) ->
   {?BT_FAILURE, BTState}.
